@@ -1,11 +1,11 @@
-from node.behaviors import Adopt
+from datetime import datetime
 from node.behaviors import Attributes
 from node.behaviors import DefaultInit
-from node.behaviors import NodeChildValidate
-from node.behaviors import Nodespaces
-from node.behaviors import Nodify
+from node.behaviors import MappingAdopt
+from node.behaviors import MappingConstraints
+from node.behaviors import MappingNode
+from node.behaviors import MappingStorage
 from node.behaviors import OdictStorage
-from node.behaviors import Storage
 from node.compat import UNICODE_TYPE
 from node.ext.ugm import Group as BaseGroupBehavior
 from node.ext.ugm import Groups as BaseGroupsBehavior
@@ -13,8 +13,8 @@ from node.ext.ugm import Ugm as BaseUgmBehavior
 from node.ext.ugm import User as BaseUserBehavior
 from node.ext.ugm import Users as BaseUsersBehavior
 from node.interfaces import IInvalidate
-from node.locking import locktree
 from node.locking import TreeLock
+from node.locking import locktree
 from node.utils import UNSET
 from odict import odict
 from plumber import Behavior
@@ -25,18 +25,19 @@ from zope.interface import implementer
 import base64
 import hashlib
 import os
+import time
 
 
 ENCODING = 'utf-8'
 
 
 @implementer(IInvalidate)
-class FileStorage(Storage):
-    """Storage behavior handling key/value pairs in a file.
+class FileStorage(MappingStorage):
+    """MappingStorage behavior handling key/value pairs in a file.
 
     Cannot contain node children. Useful for node attributes stored in a file.
     """
-    allow_non_node_children = override(True)
+    child_constraints = override(None)
     delimiter = default(':')
 
     @override
@@ -120,9 +121,9 @@ class FileStorage(Storage):
 
 
 @plumbing(
-    NodeChildValidate,
-    Adopt,
-    Nodify,
+    MappingConstraints,
+    MappingAdopt,
+    MappingNode,
     FileStorage)
 class FileAttributes(object):
     pass
@@ -168,6 +169,37 @@ class UserBehavior(BaseUserBehavior):
             self.parent()
             self.parent.parent.attrs()
 
+    @override
+    @property
+    def expired(self):
+        expires = self.expires
+        if not expires:
+            return False
+        return datetime.now() >= expires
+
+    @property
+    def expires(self):
+        ugm = self.parent.parent
+        if not ugm.user_expires_attr:
+            return None
+        expires = self.attrs.get(ugm.user_expires_attr)
+        if not expires:
+            return None
+        return datetime.fromtimestamp(float(expires))
+
+    @override
+    @expires.setter
+    def expires(self, value):
+        ugm = self.parent.parent
+        if not ugm.user_expires_attr:
+            return
+        if not value:
+            self.attrs[ugm.user_expires_attr] = ''
+            return
+        if not isinstance(value, datetime):
+            raise ValueError('Expires value must be a datetime instance')
+        self.attrs[ugm.user_expires_attr] = str(time.mktime(value.timetuple()))
+
     @default
     def add_role(self, role):
         self.parent.parent.add_role(role, self)
@@ -204,10 +236,9 @@ class UserBehavior(BaseUserBehavior):
 
 @plumbing(
     UserBehavior,
-    NodeChildValidate,
-    Nodespaces,
+    MappingConstraints,
     Attributes,
-    Nodify)
+    MappingNode)
 class User(object):
     pass
 
@@ -312,10 +343,9 @@ class GroupBehavior(BaseGroupBehavior):
 
 @plumbing(
     GroupBehavior,
-    NodeChildValidate,
-    Nodespaces,
+    MappingConstraints,
     Attributes,
-    Nodify)
+    MappingNode)
 class Group(object):
     pass
 
@@ -496,6 +526,8 @@ class UsersBehavior(SearchBehavior, BaseUsersBehavior):
         # cannot authenticate user with unset password
         if not self.storage[id]:
             return False
+        if self[id].expired:
+            return False
         return self._chk_pw(pw, self.storage[id])
 
     @default
@@ -525,11 +557,10 @@ class UsersBehavior(SearchBehavior, BaseUsersBehavior):
 
 @plumbing(
     UsersBehavior,
-    NodeChildValidate,
-    Nodespaces,
-    Adopt,
+    MappingConstraints,
+    MappingAdopt,
     Attributes,
-    Nodify,
+    MappingNode,
     FileStorage)
 class Users(object):
     pass
@@ -609,11 +640,10 @@ class GroupsBehavior(SearchBehavior, BaseGroupsBehavior):
 
 @plumbing(
     GroupsBehavior,
-    NodeChildValidate,
-    Nodespaces,
-    Adopt,
+    MappingConstraints,
+    MappingAdopt,
     Attributes,
-    Nodify,
+    MappingNode,
     FileStorage)
 class Groups(object):
     pass
@@ -636,13 +666,16 @@ class UgmBehavior(BaseUgmBehavior):
                  users_file=None,
                  groups_file=None,
                  roles_file=None,
-                 data_directory=None):
+                 data_directory=None,
+                 user_expires_attr=None):
+        # XXX: remove name and parent once using ``NodeInit`` behavior
         self.__name__ = name
         self.__parent__ = parent
         self.users_file = users_file
         self.groups_file = groups_file
         self.roles_file = roles_file
         self.data_directory = data_directory
+        self.user_expires_attr = user_expires_attr
 
     @override
     def __getitem__(self, key):
@@ -743,12 +776,11 @@ class UgmBehavior(BaseUgmBehavior):
 
 @plumbing(
     UgmBehavior,
-    NodeChildValidate,
-    Nodespaces,
-    Adopt,
+    MappingConstraints,
+    MappingAdopt,
     Attributes,
     DefaultInit,
-    Nodify,
+    MappingNode,
     OdictStorage)
 class Ugm(object):
 
